@@ -1,6 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2003, 2005-2007, 2009-2012, 2014-2017 Free Software
-   Foundation, Inc.
+   Copyright (C) 2003-2020 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -323,6 +322,9 @@ _whatnow (struct mh_whatnow_env *wh, struct action_tab *tab)
   size_t size = 0;
   struct mu_wordsplit ws;
   int wsflags = MU_WRDSF_DEFFLAGS|MU_WRDSF_COMMENT;
+
+  wh->reedit = 0;
+  wh->last_ed = NULL;
   
   do
     {
@@ -360,6 +362,8 @@ _whatnow (struct mh_whatnow_env *wh, struct action_tab *tab)
   while (rc == 0);
   if (wsflags & MU_WRDSF_REUSE)
     mu_wordsplit_free (&ws);
+  free (wh->last_ed);
+  wh->last_ed = NULL;
   free (line);
   return status;
 }
@@ -380,13 +384,87 @@ display (struct mh_whatnow_env *wh, int argc, char **argv, int *status)
 
 /* Edit action */
 static int
-edit (struct mh_whatnow_env *wh, int argc, char **argv, int *status)
+edit (struct mh_whatnow_env *wh, int argc, char **argv, int *whs)
 {
-  char *name;
+  char const *ed = wh->last_ed ? wh->last_ed : wh->editor;
+  int i, rc, status;
+  char *p;
   
-  mu_asprintf (&name, "%s-next", wh->editor);
-  invoke (name, wh->editor, argc, argv, wh->file, NULL);
-  free (name);
+  if (wh->reedit)
+    {
+      if (argc > 1)
+	ed = argv[1];
+      else
+	{
+	  char *name;
+	  char const *newed;
+	  mu_asprintf (&name, "%s-next", wh->editor);
+	  newed = mh_global_profile_get (name, NULL);
+	  free (name);
+	  if (newed)
+	    ed = newed;
+	}
+    }
+  else if (argc > 1)
+    ed = argv[1];
+
+  if (argc > 1)
+    {
+      char **xargv;
+
+      xargv = mu_calloc (argc+2, sizeof (*xargv));
+      xargv[0] = (char *)ed;
+      for (i = 1; i + 1 < argc; i++)
+	xargv[i] = argv[i+1];
+      xargv[i++] = wh->file;
+      xargv[i] = NULL;
+      rc = mu_spawnvp (xargv[0], xargv, &status);
+      free (xargv);
+    }
+  else
+    {      
+      struct mu_wordsplit ws;
+      extern char **environ;
+      ws.ws_env = (char const **)environ;
+      if (mu_wordsplit (ed, &ws,
+			MU_WRDSF_QUOTE
+			| MU_WRDSF_SQUEEZE_DELIMS
+			| MU_WRDSF_ENV 
+                        | MU_WRDSF_NOCMD))
+	{
+	  mu_error (_("cannot split line `%s': %s"), ed,
+		    mu_wordsplit_strerror (&ws));
+	  rc = MU_ERR_FAILURE;
+	}
+      else
+	{
+	  char *xargv[2];
+	  xargv[0] = wh->file;
+	  xargv[1] = NULL;
+	  if (mu_wordsplit_append (&ws, 1, xargv))
+	    {
+	      mu_error (_("cannot append arguments: %s"),
+			mu_wordsplit_strerror (&ws));
+	      rc = ENOMEM;
+	    }
+	  else
+	    rc = mu_spawnvp (ws.ws_wordv[0], ws.ws_wordv, &status);
+	  mu_wordsplit_free (&ws);
+	}
+    }
+      
+  if (rc || check_exit_status (ed, status))
+    {
+      if (wh->file)
+	mu_error (_("problems with edit--%s preserved"), wh->file);
+      else
+	mu_error (_("problems with edit"));
+    }
+
+  p = mu_strdup (ed);
+  free (wh->last_ed);
+  wh->last_ed = p;
+  wh->reedit = 1;
   
   return 0;
 }
@@ -465,9 +543,10 @@ whom (struct mh_whatnow_env *wh, int argc, char **argv, int *status)
   if (!wh->file)
     mu_error (_("no draft file to display"));
   else
-    mh_whom (wh->file, (argc == 2
-			&& (strcmp (argv[1], "-check") == 0
-			    || strcmp (argv[1], "--check") == 0)));
+    mh_whom_file (wh->file,
+		  (argc == 2
+		   && (strcmp (argv[1], "-check") == 0
+		       || strcmp (argv[1], "--check") == 0)));
   return 0;
 }
 
@@ -574,8 +653,12 @@ mh_whatnow (struct mh_whatnow_env *wh, int initial_edit)
 {
   set_default_editor (wh);
   
-  if (initial_edit)
-    mh_spawnp (wh->editor, wh->file);
+  if (initial_edit && wh->file)
+    {
+      char *argv[2] = { "edit", NULL };
+      int status;
+      edit (wh, 1, argv, &status);
+    }
 
   if (!wh->prompt)
     wh->prompt = (char*) _("What now?");
@@ -605,7 +688,6 @@ mh_whatnowproc (struct mh_whatnow_env *wh, int initial_edit, const char *prog)
   if (pid == 0)
     {
       struct mu_wordsplit ws;
-      int i;
       
       if (mu_wordsplit (prog, &ws,
 			MU_WRDSF_DEFFLAGS & ~MU_WRDSF_CESCAPES))
@@ -617,8 +699,7 @@ mh_whatnowproc (struct mh_whatnow_env *wh, int initial_edit, const char *prog)
       
       set_default_editor (wh);
       mh_whatnow_env_to_environ (wh);
-      for (i = mu_getmaxfd (); i > 2; i--)
-	close (i);
+      mu_close_fds (3);
       execvp (ws.ws_wordv[0], ws.ws_wordv);
       mu_diag_funcall (MU_DIAG_ERROR, "execvp", prog, errno);
       _exit (127);
